﻿init python:
    from copy import copy
    import pygame
    import random
    import time
    random.seed(time.perf_counter_ns())

    class mafia_player:
        def __init__(self, name, image, is_ai = True):
            self.name = name
            self.image = image
            self.dices = []
            self.points = 0
            self.is_ai = is_ai
            self.stat = "playing"
            self.think = 0

    class mafia_dice:
        def __init__(self):
            self.number = 1
            self.variation = 1
            self.rolling = 1
            self.position = None
        def roll(self):
            self.number = random.randint(1,6)
            self.variation = random.randint(1,5)
            self.rolling += 1
            if self.rolling > 10:
                self.rolling = 0
        def start_rolling(self):
            self.rolling = 1


    class mafia_class:
        def __init__(self):
            self.players = []
            self.playing_players = []
            self.turn = 0
            self.thrown_dices = []
            self.winners = []
            self.middle_positions = [
                (92, 76),
                (-68, -26),
                (60, -42),
                (-26, 66),
                (130, -125),
                (160, -32),
                (-73, -187),
                (25, -145),
                (-180, -102),
                (-200, 4),
                (-155, 98),
                (98, -241),
                (1, 177),
                (200, 91),
                (-105, 198),
                (121, 195),
                (233, -128),
                ]
        def add_players(self, *args):
            for i in args:
                self.players.append(i)
            self.playing_players = copy(self.players)
            self.start()
        def start(self):
            for player in self.players:
                player.stat = "playing"
                dices = []
                for i in range(5):
                    dice = mafia_dice()
                    dice.number = random.randint(1, 6)
                    dice.variation = random.randint(1, 5)
                    dices.append(dice)
                player.dices = dices
            self.playing_players = copy(self.players)
            self.winners = []

        def throw(self, player):
            renpy.play("mafia/throw.ogg", "sound")
            random.shuffle(self.middle_positions)
            self.thrown_dices = player.dices
            player.dices = []
            player.stat = "thrown"
            for dice, position in zip(self.thrown_dices, self.middle_positions):
                dice.position = position
                dice.start_rolling()
        def collect(self, player):
            renpy.play("mafia/collect.ogg", "sound")
            if len(self.thrown_dices) == 0:
                renpy.play("mafia/win.ogg", "sound")
                self.playing_players.remove(player)
                self.winners.append(player)
                if len(self.playing_players) == 1:
                    self.winners.append(self.playing_players[0])
                    Show("mafia_result", g = self)()
                return
            player.dices = self.thrown_dices
            player.stat = "playing"
            self.thrown_dices = []
            self.rotate()
                
        def tick(self):
            for i in self.thrown_dices:
                if i.rolling:
                    i.roll()
        def clicked(self, dice):
            if not dice.rolling:
                if dice.number == 1:
                    renpy.play("mafia/discard.ogg", "sound")
                    self.thrown_dices.remove(dice)
                elif dice.number == 6:
                    renpy.play("mafia/give.ogg", "sound")
                    self.thrown_dices.remove(dice)
                    self.playing_players[1].dices.append(dice)
            if len(self.thrown_dices) == 0:
                self.collect(self.playing_players[0])
        def rotate(self):
            p = self.playing_players.pop(0)
            self.playing_players.append(p)

        def ai(self, player):
            if player.stat == "playing":
                self.throw(player)
                return
            elif player.think < 2:
                player.think += 1
                return
            else:
                for i in self.thrown_dices:
                    if i.number in [1, 6]:
                        self.clicked(i)
                        return
                else:
                    self.collect(player)
                    player.think = 0


screen mafia_screen(g):
    modal True
    style_prefix "mafia"
    add "#000d"
    default turn_numbers = [0,1,2,3]
    default player_positions = [
        (0, 350),
        (-350, 0),
        (0, -350),
        (350, 0),
        ]
    default dice_offsets = [
        (0, 120),
        (-120, 0),
        (0, -120),
        (120, 0),
        ]
    default dice_directions = [
        "Horizontal",
        "Vertical",
        "Horizontal",
        "Vertical",
        ]
    for turn, player, position, offsets, direction in zip(turn_numbers, g.players, player_positions, dice_offsets, dice_directions):
        frame:
            background None align .5,.5 xysize 100,100 offset position
            vbox:
                frame padding 0,0:
                    if g.playing_players[0] == player:
                        background "#74d20097"
                    elif player in g.playing_players:
                        background None
                    else:
                        background "#595959"

                    add player.image align (.5,.5)
                # text player.name
                # text str(player.points)
            if direction == "Horizontal":
                hbox:
                    offset offsets
                    for i in player.dices:
                        add f"dice_{i.number}{i.variation}"
            else:
                vbox:
                    offset offsets
                    for i in player.dices:
                        add f"dice_{i.number}{i.variation}"
            if g.playing_players[0] == player and not player.is_ai:
                button:
                    offset offsets
                    if player.stat == "playing":
                            text "Throw"
                            action Function(g.throw, player)
                    elif player.stat == "thrown":
                            text "Collect"
                            action Function(g.collect, player)
    if g.playing_players[0].is_ai:
        timer .8 repeat True action Function(g.ai, g.playing_players[0])

    for dice in g.thrown_dices:
        button:
            align .5,.5 offset dice.position background None
            add f"dice_{dice.number}{dice.variation}"
            if not g.playing_players[0].is_ai:
                action Function(g.clicked, dice)
    if len(g.thrown_dices):
        timer .1 repeat True action Function(g.tick)

screen mafia_result(g):
    modal True
    style_prefix "mafia"
    add "#000d"
    default positions = [
        "Winner",
        "2nd place",
        "3rd place",
        "Loser",
    ]
    vbox spacing 40:
        text "This round's results are:"
        hbox:
            for player, position in zip(g.winners, positions):
                vbox xsize 300:
                    text position
                    add player.image align .5,.5
                    text player.name
        hbox:
            button:
                text "Retry"
                action Function(g.start), Hide("mafia_result")
            button:
                text "I've had enough."
                action Return(g.winners), Hide("mafia_result"), Hide("mafia_screen")


style mafia_text:
    align (.5,.5)
style mafia_hbox:
    align (.5,.5) spacing 10
style mafia_vbox:
    align (.5,.5) spacing 10
style mafia_button:
    align (.5,.5) background "#5e5e5ed1" padding (10,10)



# ---------------The example starts here
# define your players with name and portrait image
default mafia_aya = mafia_player("Ayame", "mafia_aya", False)
default mafia_rin = mafia_player("Ririn", "mafia_rin")
default mafia_mayo = mafia_player("Mayonnaise", "mafia_mayo")
default mafia_hana = mafia_player("Hana", "mafia_hana")

# define at least one game instance
default mafia_game_1 = mafia_class()

label mafia_example:
    show bg with dissolve
    # give the game instance a number of players between 2 and 4
    $ mafia_game_1.add_players(mafia_aya, mafia_rin, mafia_mayo, mafia_hana)
    # call the screen with the game instance
    call screen mafia_screen(mafia_game_1)

